/*******************************************************************************
  File: BtreeIndex_XIODataBuf.hxx
  Copyright (c) 2002 Bleading Edge Software Technologies, Inc.

  This Software is distributed under the open source, free software model.
  Permission to use, modify, and distribute this Software, including
  incorporating it into proprietary software, is granted without fee, 
  provided this copyright notice appear in all copies of the code.
  You are under no obligation to distribute source code which is derived from
  or uses this Software. You may not do anything however, such as copyrighting
  or claiming authorship, to prevent the continued distribution of this 
  Software under an open source distribution model.
  This Software is distributed in the hope that it will be useful, but it is
  provide "as is", WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

//$-----------------------------------------------------------------------------
Author: Jack W. Reeves

//!----------------------------------------------------------------------------|
Description:
    The template implementation file for the BtreeIndex class.
  
//#----------------------------------------------------------------------------|
Notes:
  
//%****************************************************************************/

// My header
#include "BtreeIndex.h"


namespace Util {

//
// XIODataStreambuf - accepts all base class default operations
//
template<typename Key, typename Compare, typename KeyTraits>
BtreeIndex_XIODataStreambuf<Key,Compare,KeyTraits>::BtreeIndex_XIODataStreambuf(
            XDR_Char* buf, size_t size, size_t len)
{
    // Set up the get and put areas
    setg(buf, buf, buf+len);
    setp(buf+len, buf+size);
}


template<typename Key, typename Compare, typename KeyTraits>
BtreeIndex_XIODataStreambuf<Key,Compare,KeyTraits>::~BtreeIndex_XIODataStreambuf()
{
    // empty
}


template<typename Key, typename Compare, typename KeyTraits>
BtreeIndex_XIODataStreambuf<Key,Compare,KeyTraits>::pos_type 
BtreeIndex_XIODataStreambuf<Key,Compare,KeyTraits>::seekoff(
        off_type off, std::ios_base::seekdir dir, std::ios_base::openmode which)
{
    if (which & std::ios_base::in and
        which & std::ios_base::out and 
        dir & std::ios_base::cur) return -1;  // not valid

    pos_type rtn;
    if (which & std::ios_base::in) {
        int curoff = gptr() - eback();
        int newoff;
        if (dir & std::ios_base::beg) newoff = 0;
        else if (dir & std::ios_base::cur) newoff = curoff;
        else newoff = egptr() - eback();
        if (newoff + off < 0 or newoff + off > egptr() - eback()) return -1;
        if (newoff + off != curoff) gbump(newoff - curoff + off);
        rtn = newoff + off;
    }
    if (which & std::ios_base::out) {
        int curoff = pptr() - pbase();
        int newoff;
        if (dir & std::ios_base::beg) newoff = 0;
        else if (dir & std::ios_base::cur) newoff = curoff;
        else newoff = epptr() - pbase();
        if (newoff + off < 0 or newoff + off > epptr() - pbase()) return -1;
        if (newoff + off != curoff) pbump(newoff - curoff + off);
        rtn = newoff + off;
    }
    return rtn;
}


template<typename Key, typename Compare, typename KeyTraits>
BtreeIndex_XIODataStreambuf<Key,Compare,KeyTraits>::pos_type 
BtreeIndex_XIODataStreambuf<Key,Compare,KeyTraits>::seekpos(
        pos_type pos, std::ios_base::openmode which)
{
    return seekoff(pos, std::ios_base::beg, which);
}


//
// XOutDataBuf - the XDR_Stream class
//
template<typename Key, typename Compare, typename KeyTraits>
BtreeIndex_XOutDataBuf<Key,Compare,KeyTraits>::BtreeIndex_XOutDataBuf(
            oXDR_Stream& xstrm,
            XDR_Char* buf, 
            size_t size)
  : _refstrm(xstrm),
    _xstreambuf(buf, size, 0)
{
    // Initialize the base class
    init(&_xstreambuf);

    // Copy the formating, and polymorphic support information
    copyfmt(xstrm);
    obj2id()  = xstrm.obj2id();
    type2id() = xstrm.type2id();
}



template<typename Key, typename Compare, typename KeyTraits>
BtreeIndex_XOutDataBuf<Key,Compare,KeyTraits>::~BtreeIndex_XOutDataBuf()
{
    // copy the formating data back
    _refstrm.copyfmt(*this);
    _refstrm.obj2id()  = obj2id();
    _refstrm.type2id() = type2id();
}


//
// XInDataBuf - the XDR_Stream class
//
template<typename Key, typename Compare, typename KeyTraits>
BtreeIndex_XInDataBuf<Key,Compare,KeyTraits>::BtreeIndex_XInDataBuf(
            iXDR_Stream& xstrm,
            XDR_Char* buf, 
            size_t len)
  : _refstrm(xstrm),
    _xstreambuf(buf, len, len)
{
    // Initialize the base class
    init(&_xstreambuf);

    // Copy the formating, and polymorphic support information
    copyfmt(xstrm);
    id2obj()  = xstrm.id2obj();
    id2type() = xstrm.id2type();
}



template<typename Key, typename Compare, typename KeyTraits>
BtreeIndex_XInDataBuf<Key,Compare,KeyTraits>::~BtreeIndex_XInDataBuf()
{
    // copy the formating data back
    _refstrm.copyfmt(*this);
    _refstrm.id2obj()  = id2obj();
    _refstrm.id2type() = id2type();
}


} //> namespace
