/*******************************************************************************
  File: platform.h

//$----------------------------------------------------------------------------|
$Header: $
  Author: Jack W. Reeves

//!----------------------------------------------------------------------------|
Description:
  Contains some 'defacto' languge standard extensions that I use all the
  time.
  
//#----------------------------------------------------------------------------|
Note:
  This file is organized first by compiler since that is typically the easiest
  thing to detect, and the platform flag may differ between compilers on the 
  same platform. Next comes any platform specific stuff when the compiler
  is available on more than one platform. Then comes the different compiler
  version information. 
  Only a few compilers are available on multiple platforms.

//%****************************************************************************/

#ifndef UTIL_PLATFORM_H
#define UTIL_PLATFORM_H

#if defined (__GNUG__)

// I should be able to declare a const, but NOOOOO not with GNU
#define null 0

// Get the byte order
#if defined(__linux)
#include <endian.h>
#elif defined(__sun)
#define LITTLE_ENDIAN 1234
#define BIG_ENDIAN 4321
#if defined(_BIG_ENDIAN)
#define BYTE_ORDER BIG_ENDIAN
#else
#define BYTE_ORDER LITTLE_ENDIAN
#endif
#else
#error "No byte order available for this GNU platform"
#endif


#if __GNUC__ == 2 && __GNUC_MINOR__ < 96
// we have namespaces, but the compiler ignores 'std' unless -fhonor-std is used
// if we define it to be nothing, it screws things up.
// #define std
// The GNU compiler (egcs-1.2.2) does not recognize the ISO646 keywords
// On the other hand, some well meaning idiot has put an #ifndef __cplusplus
// around the #defines in iso646.h. Rather than diddle with the header (which
// is wrong), I will just define them here
#define and &&
#define and_eq  &=
#define bitand  &
#define bitor   |
#define compl   ~
#define not !
#define not_eq  !=
#define or  ||
#define or_eq   |=
#define xor ^
#define xor_eq  ^=

// keyword 'export' not available
#define export

// Things that do not work
#define EXPORT_NOT_SUPPORTED 1
#define NESTED_TEMPLATE_DEFINITION_NOT_SUPPORTED 1
#define NO_FULL_STANDARD_LIBRARY 1

#elif __GNUC__ == 3

// Missing keywords
#define export

// Things that do not work
#define EXPORT_NOT_SUPPORTED 1

#else

#error "Unrecognized GNU compiler"

#endif


#elif defined(_MSC_VER)
// Microsoft fixes

// I hate to use NULL, but I hate magic numbers even more
const long null = 0L;

// Define all the keywords
// I use to include <iso.646.h> here, but the latest version of the 
// Dinkumware library works like the GNU library mentioned above.
// So I define the keywords myself.
#define and &&
#define and_eq  &=
#define bitand  &
#define bitor   |
#define compl   ~
#define not !
#define not_eq  !=
#define or  ||
#define or_eq   |=
#define xor ^
#define xor_eq  ^=

// Other missing keywords
#define export

// Things that do not work
#define EXPORT_NOT_SUPPORTED 1
#define NESTED_TEMPLATE_DEFINITION_NOT_SUPPORTED 1
#define NO_FULL_STANDARD_LIBRARY 1 // close, but the compiler isn't quite up to it


// Define min & max  - make sure to define NOMINMAX for the project
#ifndef NOMINMAX
#define NOMINMAX 1
#endif
#define min _cpp_min
#define max _cpp_max

// disable the warning about names being too long for debugging information
#pragma warning( disable: 4786)

#elif defined(_AIX) 

// null
const long null = 0L;

#if __IBMCPP__ < 400
// some new keywords
#define and &&
#define and_eq  &=
#define bitand  &
#define bitor   |
#define compl   ~
#define not !
#define not_eq  !=
#define or  ||
#define or_eq   |=
#define xor ^
#define xor_eq  ^=

// Other missing keywords
#define explicit
#define typename
#define export

// Get the byte order
#include <sys/machine.h>

// Things that do not work
#define EXPORT_NOT_SUPPORTED 1
#define NESTED_TEMPLATE_DEFINITION_NOT_SUPPORTED 1
#define NO_FULL_STANDARD_LIBRARY 1

#include <ilanglvl.hpp> // defines bool

// no namespaces yet
#define std

// some new cast notation
#define STATIC_CAST(X,Y) ((X)(Y))
#define DYNAMIC_CAST(X,Y) ((X)(Y))
#define CONST_CAST(X,Y) ((X)(Y))
#define REINTERPRET_CAST(X,Y) ((X)(Y))

#endif

#elif defined(__hpux)

// null
const long null = 0L;

// standard library is not in namespace std yet
#define std

// Missing keywords
#define export

// Define the BYTE_ORDER
#define LITTLE_ENDIAN 1234
#define BIG_ENDIAN 4321
#define BYTE_ORDER BIG_ENDIAN

// Things that do not work
#define EXPORT_NOT_SUPPORTED 1
#define NO_FULL_STANDARD_LIBRARY 1

#elif defined(__SUNPRO_CC)

// null
#define null 0

// Missing keywords
#define export

// Define the BYTE_ORDER
#define LITTLE_ENDIAN 1234
#define BIG_ENDIAN 4321
#define BYTE_ORDER BIG_ENDIAN

// Things that do not work
#define EXPORT_NOT_SUPPORTED 1
#define NESTED_TEMPLATE_DEFINITION_NOT_SUPPORTED 1

#else

#error "No compiler section in platform.h"

#endif

#endif
