/*******************************************************************************
  File: XDRStream_reader.hxx
  Copyright (c) 1999 Bleading Edge Software Technologies, Inc.

  This Software is distributed under the open source, free software model.
  Permission to use, modify, and distribute this Software, including
  incorporating it into proprietary software, is granted without fee, 
  provided this copyright notice appear in all copies of the code.
  You are under no obligation to distribute source code which is derived from
  or uses this Software. You may not do anything however, to prevent the 
  continued distribution of this Software under an open source distribution
  model. This includes, but is not limited to, such things as copyrighting
  or claiming authorship of the Software.
  This Software is distributed in the hope that it will be useful, but it is
  provide "as is", WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

//$-----------------------------------------------------------------------------
  Author: Jack W. Reeves

//!----------------------------------------------------------------------------|
Description:
  Implements the exported templates of iXDR_Stream.
  
//#----------------------------------------------------------------------------|
Notes:
  
//%****************************************************************************/
// My headers
#include "XDRStream.h"

namespace Util {

template<typename T>
ptrdiff_t
iXDR_Stream::get_array(T* a, ptrdiff_t l)
{
    sentry qx(*this);
    if (not qx) return 0;

    T* p = a;
    while (l > 0) {
        *this >> *p;
        if (not good()) break;
        ++p;
        --l;
    }
    return p - a;
}


template<typename T>
ptrdiff_t
iXDR_Stream::vget_array(T* a, ptrdiff_t max)
{
    sentry qx(*this);
    if (not qx) return 0;

    unsigned long len;
    *this >> len;
    if (fail()) return 0;
    if (max != -1 and len > max) {
        setstate(failbit);
        return 0;
    }
    return get_array(a, len);
}


template<typename T>
iXDR_Stream&
operator>>(iXDR_Stream& xis, std::vector<T>& v)
{
    iXDR_Stream::sentry qx(xis);
    if (not qx) return xis;
    
    v.clear();
    unsigned long l;
    xis >> l;
    if (xis.fail()) return xis;
    
    T t;
    while (l-- > 0) {
        xis >> t;
        if (xis.fail()) break;
        v.push_back(t);
    }
    if (xis.fail()) v.clear();
    return xis;
}


template<typename T>
std::pair<T*, ptrdiff_t>
vget_array(iXDR_Stream& ixs, ptrdiff_t max)
{
    std::pair<T*, ptrdiff_t> nothing(0, 0);
    iXDR_Stream::sentry qx(ixs);
    if (not qx) return nothing;

    long l;
    ixs >> l;
    if (ixs.fail() or l == 0) return nothing;
    if (max != -1 and l > max) {
        ixs.setstate(std::ios_base::failbit);
        return nothing;
    }

    T* p = new T[l];
    xis.get_array(p, l);
    if (xis.fail()) {
        delete[] p;
        return nothing;
    }
    return std::make_pair(p, l);
}


template<typename T>
T* get_optional(iXDR_Stream& ixs)
{
    long l;
    T* p = null;
    ixs >> l;
    if (l) {
        p = new T;
        ixs >> *p;
    }
    return p;
}
        

} //> namespace Util
