/*
   +------------------------+
   | Cleanup Framework in C |`
   | ~~~~~~~~~~~~~~~~~~~~~~ |`
   | Stephane GRAD          |`
   | (2001)                 |`
   +------------------------+`
    ``````````````````````````

   +----------------+
   | Implementation |
---+----------------+-----------------------------------------------------------
*/

/* should be passed as arg to the compiler
 */
//#define DEBUGCL

#include <assert.h>

#ifdef DEBUGCL
#include <stdio.h> // printf
#define DEBUGCL_PRINT printf
#endif /* DEBUGCL */

#include "cleanup.h"


#ifdef DEBUGCL

#ifndef DEBUGCL_PRINT
#error "Don't forget to #define DEBUGCL_PRINT in cleanup.c file"
#endif

#define DEBUGCL_WHERE_FORMAT "%s, %d"

#define DEBUGCL_WHERE() DEBUGCL_PRINT(DEBUGCL_WHERE_FORMAT DEBUGCL_EXTRA_ARGS);

void DEBUGCL_DUMP(tCleanupStack* aStack)
{
    DEBUGCL_PRINT("Cleanup Stack ("DEBUGCL_WHERE_FORMAT")\n", aStack->filename, aStack->lineno);
    {
        unsigned int n = aStack->index;
        while (n--)
        {
            tCleanupElm* t;
            t = &aStack->stack[n];
            DEBUGCL_PRINT("%d: (0x%x, 0x%x) "DEBUGCL_WHERE_FORMAT"\n",
                n, t->ptr, t->clean, t->filename, t->lineno);
        }
    }
}

#endif /* DEBUGCL */

void _cl_push(tCleanupStack* aStack, void* aPtr, void (*aClean)(void*)
DEBUGCL_EXTRA_PARAMS)
{
    tCleanupElm* t;

#ifndef DEBUGCL
    assert(aStack->index < aStack->max);
#else
    if (!(aStack->index < aStack->max))
    {
        DEBUGCL_WHERE();
        DEBUGCL_PRINT(": Cleanup Stack overflow\n");
        DEBUGCL_DUMP(aStack);
        assert(0);
    }
#endif
    
    t = &aStack->stack[aStack->index];
    t->ptr = aPtr;
    t->clean = aClean;
#ifdef DEBUGCL
    t->filename = aFileName;
    t->lineno = aLineNo;
#endif
    ++aStack->index;
}

void _cl_pop(tCleanupStack* aStack, unsigned int n
DEBUGCL_EXTRA_PARAMS)
{
#ifndef DEBUGCL
    assert(n <= aStack->index);
#else
    if (!(n <= aStack->index))
    {
        DEBUGCL_WHERE();
        DEBUGCL_PRINT(": Cleanup Stack underflow (pop %d)\n", n);
        DEBUGCL_DUMP(aStack);
        assert(0);
    }
#endif

    while (n--)
    {
        tCleanupElm* t;
        --aStack->index;
        t = &aStack->stack[aStack->index];
        t->clean(t->ptr);
    }
}

void _cl_popa(tCleanupStack* aStack
DEBUGCL_EXTRA_PARAMS)
{
    _cl_pop(aStack, aStack->index
        DEBUGCL_EXTRA_ARGS);
}
